#ifndef WATCH_ACCESS_H
#define WATCH_ACCESS_H

#include "watch_state.h"

#include <boost/shared_ptr.hpp>

class watch_access
{
public:
   // Virtual member function templates are not possible. Instead, intercept the call chain by capturing the 
   // template argument in a non-virtual function, create the new state instance and continue through the virtual function mechanism.
   template<class new_state>
   void change_state_to()
   {
      watch_state_ptr created_state(new new_state);
      
      change_state_to(created_state);
   }

protected:
   ~watch_access() {}
   
   typedef boost::shared_ptr<watch_state> watch_state_ptr;
   
private:
   // A template method where the deriving class performs the actual state management.
   virtual void change_state_to(watch_state_ptr new_state) = 0;
};

#endif
