#ifndef WATCH_ACCESS_H
#define WATCH_ACCESS_H

#include "flyweight_factory.h"
#include "watch_state.h"

class watch_access
{
   typedef flyweight_factory<watch_state> state_factory;
   state_factory factory;
   
public:
   // Virtual member function templates are not possible. Instead, intercept the call chain by capturing the 
   // template argument in a non-virtual function, retrieve the flyweight and continue through the virtual function mechanism.
   template<class new_state>
   void change_state_to()
   {
      change_state_to(factory.get_flyweight<new_state>());
   }

protected:
   ~watch_access() {}
   
   typedef state_factory::flyweight_ptr watch_state_ptr;
   
private:
   // A template method where the deriving class performs the actual state management.
   virtual void change_state_to(watch_state_ptr new_state) = 0;
};

#endif
